-- Change this to use the name of some database. The current default is "mysql".
USE mysql;

-- Set this to the name of the database
SET @database_name = 'fds';

-- Set this to the name of the user
SET @user_name = 'fds_user';

-- Set this to the user's password
SET @user_password = 'admin';

-- Set this to True if you would like the procedure to generate and execute the script; False will cause the procedure to just print the script
SET @execute = False;

-- 
-- Loader Database Setup procedure
-- 
DROP PROCEDURE IF EXISTS loader_db_setup;

DELIMITER //
CREATE PROCEDURE loader_db_setup (IN database_name VARCHAR(50), IN user_name VARCHAR(50), IN user_password VARCHAR(50), IN execute BOOL)
BEGIN
	SET @create_database_query         = CONCAT('CREATE DATABASE ', database_name);
	SET @create_user_query             = CONCAT('CREATE USER ''', user_name, ''' IDENTIFIED BY ''', REPLACE(user_password, '''', ''''''), '''');
	SET @grant_select_query            = CONCAT('GRANT SELECT ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_drop_query              = CONCAT('GRANT DROP ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_insert_query            = CONCAT('GRANT INSERT ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_delete_query            = CONCAT('GRANT DELETE ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_create_query            = CONCAT('GRANT CREATE ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_update_query            = CONCAT('GRANT UPDATE ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_alter_query             = CONCAT('GRANT ALTER ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_index_query             = CONCAT('GRANT INDEX ON ', database_name, '.* TO ''', user_name, '''@''%''');
	SET @grant_create_temp_table_query = CONCAT('GRANT CREATE TEMPORARY TABLES ON ', database_name, '.* TO ''', user_name, '''@''%''');
	          
	IF execute
	THEN
		IF NOT EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME=database_name)
		THEN
			SELECT CONCAT('RUNNING QUERY: ', @create_database_query);
			PREPARE create_database_stmt FROM @create_database_query;
			EXECUTE create_database_stmt;
			DEALLOCATE PREPARE create_database_stmt;
		END IF;

		IF NOT EXISTS (SELECT 1 FROM mysql.user WHERE USER = user_name)
		THEN
			SELECT CONCAT('RUNNING QUERY: ', @create_user_query);
			PREPARE create_user_stmt FROM @create_user_query;
			EXECUTE create_user_stmt; 
			DEALLOCATE PREPARE create_user_stmt;
		END IF;

		SELECT CONCAT('RUNNING QUERY: ', @grant_select_query);
		PREPARE grant_select_stmt FROM @grant_select_query;
		EXECUTE grant_select_stmt;
		DEALLOCATE PREPARE grant_select_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_drop_query);
		PREPARE grant_drop_stmt FROM @grant_drop_query;
		EXECUTE grant_drop_stmt;
		DEALLOCATE PREPARE grant_drop_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_insert_query);
		PREPARE grant_insert_stmt FROM @grant_insert_query;
		EXECUTE grant_insert_stmt;
		DEALLOCATE PREPARE grant_insert_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_delete_query);
		PREPARE grant_delete_stmt FROM @grant_delete_query;
		EXECUTE grant_delete_stmt;
		DEALLOCATE PREPARE grant_delete_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_create_query);
		PREPARE grant_create_stmt FROM @grant_create_query;
		EXECUTE grant_create_stmt;
		DEALLOCATE PREPARE grant_create_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_update_query);
		PREPARE grant_update_stmt FROM @grant_update_query;
		EXECUTE grant_update_stmt;
		DEALLOCATE PREPARE grant_update_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_alter_query);
		PREPARE grant_alter_stmt FROM @grant_alter_query;
		EXECUTE grant_alter_stmt;
		DEALLOCATE PREPARE grant_alter_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_index_query);
		PREPARE grant_index_stmt FROM @grant_index_query;
		EXECUTE grant_index_stmt;
		DEALLOCATE PREPARE grant_index_stmt;
		SELECT CONCAT('RUNNING QUERY: ', @grant_create_temp_table_query);
		PREPARE grant_create_temp_table_stmt FROM @grant_create_temp_table_query;
		EXECUTE grant_create_temp_table_stmt;
		DEALLOCATE PREPARE grant_create_temp_table_stmt;
	ELSE
		SELECT CONCAT(@create_database_query, ';\n',
		              @create_user_query, ';\n',
		              @grant_select_query, ';\n',
		              @grant_drop_query, ';\n',
		              @grant_insert_query, ';\n',
		              @grant_delete_query, ';\n',
		              @grant_create_query, ';\n',
		              @grant_update_query, ';\n',
		              @grant_alter_query, ';\n',
		              @grant_index_query, ';\n',
		              @grant_create_temp_table_query, ';') AS 'setup_script';
	END IF;
END//

DELIMITER ;

CALL loader_db_setup(@database_name, @user_name, @user_password, @execute);

DROP PROCEDURE IF EXISTS loader_db_setup;