-- Set this to the name of the database
DECLARE @database_name VARCHAR(50) = 'fds';

-- Set this to True if the user being created is windows authenticated
DECLARE @windows_auth BIT = 'False';

-- Set this to the name of the user. Include the domain name if the user is windows authenticated (e.g. DOMAIN\USER)
DECLARE @user_name VARCHAR(50) = 'fds_user';

-- Set this to the user's password (only if the user is not windows authenticated). Ensure that the password is complex enough to meet your Windows policy requirements.
DECLARE @user_password VARCHAR(50) = 'admin';

-- Set this to NULL for on prem instances or azure/rds as appropriate
DECLARE @cloud_type VARCHAR(20) = NULL

-- Set this to the path where the database and log (mdf & ldf) files will be stored
DECLARE @database_file_path VARCHAR(500) = ''
SET @database_file_path = CAST(SERVERPROPERTY('instancedefaultdatapath') AS VARCHAR(500)) -- This default path is valid for SQL Server 2012 and up. Delete this line if you are on a lower version or don't want to use the default

-- Set this to the initial size of the database. This can be number of bytes or higher designations (e.g., 1MB, 1GB, etc.).
DECLARE @database_initial_size VARCHAR(50) = '1GB'

-- Set this to True if you would like the procedure to generate and execute the script; False will cause the procedure to just print the script
-- This setting will be ignored if cloud_type = azure because this script cannot be automatically executed on azure sql environments
DECLARE @execute BIT = 'False';

-- 
-- Loader Database Setup procedure
-- 
DECLARE @full_version VARCHAR(50) = CAST(SERVERPROPERTY('productversion') AS VARCHAR(50));
DECLARE @version_number INT = CAST(LEFT(@full_version, charindex('.', @full_version) - 1) AS INT);
SET @cloud_type = LOWER(@cloud_type)

DECLARE @create_database_query NVARCHAR(MAX) = 
'CREATE DATABASE [' + @database_name + ']' + IIF(@cloud_type IS NULL, '
ON PRIMARY (NAME = ''' + @database_name + ''',
            FILENAME = ''' + @database_file_path + '\\' + @database_name + '.mdf'',
            SIZE = ' + @database_initial_size + ',
            MAXSIZE = UNLIMITED,
            FILEGROWTH = 10%
            )
LOG ON (NAME = ''' + @database_name + '_log'',
        FILENAME = ''' + @database_file_path + '\\' + @database_name + '_log.ldf'',
        SIZE = 5GB,
        MAXSIZE = UNLIMITED,
        FILEGROWTH = 10%
        );',';');

DECLARE @alter_recovery_model_query NVARCHAR(MAX) = 'ALTER DATABASE [' + @database_name + '] SET RECOVERY SIMPLE;';
DECLARE @create_user_query NVARCHAR(MAX) = IIF(@cloud_type <> 'azure', 'USE [' + @database_name + ']; ', '') + 'CREATE USER [' + @user_name + '] FOR LOGIN [' + @user_name + '] WITH DEFAULT_SCHEMA=[dbo];';
DECLARE @grant_create_schema_query NVARCHAR(MAX) = IIF(@cloud_type <> 'azure', 'USE [' + @database_name + ']; ', '') + 'GRANT CREATE SCHEMA TO [' + @user_name + '];';
DECLARE @grant_create_table_query NVARCHAR(MAX) = IIF(@cloud_type <> 'azure', 'USE [' + @database_name + ']; ', '') + 'GRANT CREATE TABLE TO [' + @user_name + '];';
DECLARE @grant_view_any_database_query NVARCHAR(MAX) = IIF(@cloud_type <> 'azure', 'USE [' + @database_name + ']; ', '') + 'GRANT VIEW ANY DATABASE TO [' + @user_name + '];';
DECLARE @alter_role_db_datawriter_query NVARCHAR(MAX) = IIF(@cloud_type <> 'azure', 'USE [' + @database_name + ']; ', '') + 'ALTER ROLE db_datawriter ADD MEMBER [' + @user_name + '];';

DECLARE @create_login_query NVARCHAR(MAX) = NULL;

IF @windows_auth = 1
BEGIN
	SET @create_login_query = 'CREATE LOGIN [' + @user_name + '] FROM WINDOWS' + IIF(@cloud_type <> 'azure', ' WITH DEFAULT_DATABASE=[' + @database_name + '];', ';');
END
ELSE
BEGIN
	SET @create_login_query = 'CREATE LOGIN [' + @user_name + '] WITH PASSWORD=N''' + REPLACE(@user_password, '''', '''''') + '''' + IIF(@cloud_type <> 'azure', ', DEFAULT_DATABASE=[' + @database_name + '];', ';');
END

DECLARE @add_bulk_admin_query NVARCHAR(MAX) = NULL;

IF @version_number <= 10 -- v10 = SQL Server 2008
BEGIN
	SET @add_bulk_admin_query = 'EXEC master..sp_addsrvrolemember @loginame = ''' + @user_name + ''', @rolename = N''bulkadmin'';';
END
ELSE
BEGIN
	SET @add_bulk_admin_query = 'ALTER SERVER ROLE [bulkadmin] ADD MEMBER [' + @user_name + '];';
END

IF @execute = 1 AND @cloud_type <> 'azure'
BEGIN
	DECLARE @info VARCHAR(MAX) = NULL;

	DECLARE @check_database_query NVARCHAR(MAX) = 'SELECT 1 FROM master.dbo.sysdatabases WHERE name = @database_name';
	DECLARE @check_database_result TABLE (flag INT);
	INSERT INTO @check_database_result
	EXEC sp_executesql @check_database_query, N'@database_name VARCHAR(50)', @database_name = @database_name;

	IF NOT EXISTS (SELECT * FROM @check_database_result)
	BEGIN
		SET @info = 'RUNNING QUERY: ' + @create_database_query;
		PRINT @info;
		EXEC sp_executesql @create_database_query;
	END

	IF @cloud_type <> 'rds'
	BEGIN
		SET @info = 'RUNNING QUERY: ' + @alter_recovery_model_query;
		PRINT @info;
		EXEC sp_executesql @alter_recovery_model_query;
	END

	DECLARE @check_login_query NVARCHAR(MAX) = 'SELECT 1 FROM sys.server_principals WHERE name = @user_name';
	DECLARE @check_login_result TABLE (flag INT);
	INSERT INTO @check_login_result
	EXEC sp_executesql @check_login_query, N'@user_name VARCHAR(50)', @user_name = @user_name;

	IF NOT EXISTS (SELECT * FROM @check_login_result)
	BEGIN
		SET @info = 'RUNNING QUERY: ' + @create_login_query;
		PRINT @info;
		EXEC sp_executesql @create_login_query;
	END

	DECLARE @check_user_query NVARCHAR(MAX) = 'USE [' + @database_name + ']; SELECT 1 FROM sys.database_principals WHERE name = @user_name';
	DECLARE @check_user_result TABLE (flag INT);
	INSERT INTO @check_user_result
	EXEC sp_executesql @check_user_query, N'@user_name VARCHAR(50)', @user_name = @user_name;

	IF NOT EXISTS (SELECT * FROM @check_user_result)
	BEGIN
		SET @info = 'RUNNING QUERY: ' + @create_user_query;
		PRINT @info;
		EXEC sp_executesql @create_user_query;
	END

	SET @info = 'RUNNING QUERY: ' + @grant_create_schema_query;
	PRINT @info;
	EXEC sp_executesql @grant_create_schema_query;

	SET @info = 'RUNNING QUERY: ' + @grant_create_table_query;
	PRINT @info;
	EXEC sp_executesql @grant_create_table_query;

	SET @info = 'RUNNING QUERY: ' + @grant_view_any_database_query;
	PRINT @info;
	EXEC sp_executesql @grant_view_any_database_query;

	IF @cloud_type <> 'rds'
	BEGIN
		SET @info = 'RUNNING QUERY: ' + @add_bulk_admin_query;
		PRINT @info;
		EXEC sp_executesql @add_bulk_admin_query;
	END
	ELSE
	BEGIN
		SET @info = 'RUNNING QUERY: ' + @alter_role_db_datawriter_query;
		PRINT @info;
		EXEC sp_executesql @alter_role_db_datawriter_query;
	END
END
ELSE
BEGIN
	IF @cloud_type = 'azure'
	BEGIN
		PRINT 'Manually run the following queries:';
		SET @create_login_query = @create_login_query + ' -- Run this while connected to the master DB';
		SET @create_user_query = @create_user_query + ' -- Run this while connected to the ' + @database_name + ' DB';
		SET @grant_create_schema_query = @grant_create_schema_query + ' -- Run this while connected to the ' + @database_name + ' DB';
		SET @grant_create_table_query = @grant_create_table_query + ' -- Run this while connected to the ' + @database_name + ' DB';
		SET @grant_view_any_database_query = @grant_view_any_database_query + ' -- Run this while connected to the ' + @database_name + ' DB';
		SET @alter_role_db_datawriter_query = @alter_role_db_datawriter_query + ' -- Run this while connected to the ' + @database_name + ' DB';
	END
	ELSE
	BEGIN
		SET @create_database_query = @create_database_query + ' -- Run this first';
		PRINT @create_database_query;
		IF @cloud_type <> 'rds'
		BEGIN
			PRINT @alter_recovery_model_query;
		END
	END
	PRINT @create_login_query;
	PRINT @create_user_query;
	PRINT @grant_create_schema_query;
	PRINT @grant_create_table_query;
	PRINT @grant_view_any_database_query;
	IF @cloud_type IS NULL
	BEGIN
		PRINT @add_bulk_admin_query;
	END
	ELSE
	BEGIN
		PRINT @alter_role_db_datawriter_query;
	END
END
