DO $$
DECLARE 
-- Set this to the name of the database
database_name VARCHAR(50) = 'fds';

-- Set this to the name of the user
user_name VARCHAR(50) = 'fds_user';

-- Set this to the user's password
user_password VARCHAR(50) = 'admin';

-- Set this to True if you would like the procedure to generate and execute the script; False will cause the procedure to just print the script
execute_ind BOOLEAN = FALSE;

-- Set this to the user name of the admin/super user account. This is needed in order to re-connect to the DB to create the database
-- Only needed if you are going to automatically execute the script.
admin_name VARCHAR(50) = 'postgres';

-- Set this to the password of the admin/super user account. This is needed in order to re-connect to the DB to create the database
-- Only needed if you are going to automatically execute the script.
admin_password VARCHAR(50) = 'admin_pass';

-- 
-- Loader Database Setup procedure
-- 
BEGIN

DECLARE
	create_database_query TEXT;
	create_user_query TEXT;
	grant_create_query TEXT;
	extension_exists BOOLEAN = FALSE;
	extension_installed BOOLEAN = FALSE;
BEGIN
	create_database_query = 'CREATE DATABASE ' || database_name || ' ENCODING ''UTF8''';
	create_user_query = 'CREATE USER ' || user_name || ' WITH password ''' || REPLACE(user_password, '''', '''''')  || '''';
	grant_create_query = 'GRANT CREATE ON DATABASE ' || database_name || ' TO ' || user_name;
	
	IF execute_ind
	THEN
		SELECT COUNT(*) = 1, COUNT(installed_version) = 1 INTO extension_exists, extension_installed FROM pg_available_extensions WHERE name = 'dblink';
		IF extension_exists
		THEN
			IF NOT extension_installed
			THEN
				CREATE EXTENSION dblink;
			END IF;
		ELSE 
			RAISE 'Required dblink extension is not available. This is needed to automate the creation of databases.';
		END IF;
		
		IF NOT EXISTS (SELECT 1 FROM pg_database WHERE datname = database_name)
		THEN
			RAISE NOTICE 'RUNNING QUERY: %', create_database_query;
			PERFORM dblink_exec('user=' || admin_name || ' password=' || admin_password || ' dbname=' || CURRENT_DATABASE(), create_database_query);
		END IF;
		
		IF NOT EXISTS (SELECT 1 FROM pg_roles WHERE rolname = user_name)
		THEN
			RAISE NOTICE 'RUNNING QUERY: %', create_user_query;
			EXECUTE create_user_query;
		END IF;
		
		RAISE NOTICE 'RUNNING QUERY: %', grant_create_query;
		EXECUTE grant_create_query;
		
		-- only drop the extension if it was installed as part of this script; existing installations will not be affected
		IF NOT extension_installed
		THEN
			DROP EXTENSION dblink;
		END IF;
	ELSE
		RAISE NOTICE E'SETUP SCRIPT:\n%', create_database_query || E';\n' || create_user_query || E';\n' || grant_create_query || ';';
	END IF;
END;

END $$;